# -*- coding: utf-8 -*-
## GrufSpaces
## Group environments for elaborating and exchanging information
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
from Products.funittest import DataProvider
from Products.funittest import dataprovider
from Products.funittest import logical

class AreaGroupSpaces(DataProvider):

    def populate(self):
        context = dataprovider.navigation.current    
        for groupspace in self.all():
            self.instantiate(context, groupspace)
            self.edit(groupspace)
        
default = {'id':'groupspace',
           'title':'A GroupSpace',
           'type_name':'GroupSpace'}
key="title"
areagroupspaces = AreaGroupSpaces("GrufSpaces", default, key)
provide = areagroupspaces.provide
provide(id='areagroupspace1',
        title='GroupSpace 1 with Areas')
provide(id='areagroupspace2',
        title='GroupSpace 2 with Areas')
provide(id='areagroupspace',
        title='GroupSpace with Areas')
        
class SetUpAreaGroupSpace:
    def __init__(self, groupspace):
        self.groupspace=groupspace
        
    def setup(self):
        # Store the current context
        self.context = dataprovider.navigation.current
        
        # Always logout before logging in
        logical.application.logout()
        user = dataprovider.user.take("Sample Manager")
        logical.application.login(user)
        
        # We want standard GroupSpaces
        config = dataprovider.configuration.filtered_choice(areas=1)
        logical.configlet.configure(config)
        
        # Always logout before logging in
        logical.application.logout()
        user1 = dataprovider.user.take("Sample Manager")
        logical.application.login(user1)
                
        dataprovider.navigation.current = self.context

        logical.folder.delete_using_url(self.groupspace)
        logical.folder.add(self.groupspace)
        logical.content.edit(self.groupspace)
        logical.content.save(self.groupspace)
        
        dataprovider.groupspaceuser.populate()
        
        logical.navigation.up()
         
    def goto(self):
        dataprovider.navigation.current = self.context
        logical.navigation.down(self.groupspace)
        
areagroupspace = dataprovider.areagroupspaces.take('GroupSpace with Areas')
dataprovider.context.provide(id=areagroupspace['title'],
                             setup=SetUpAreaGroupSpace(areagroupspace),) 

class SetUpAreaGroupSpaceArea:
    def __init__(self, groupspace, area):
        self.groupspace=groupspace
        self.area=area
        
    def setup(self):
        # Store the current context
        self.context = dataprovider.navigation.current
        
        # Always logout before logging in
        logical.application.logout()
        user = dataprovider.user.take("Sample Manager")
        logical.application.login(user)
        
        # We want an area GroupSpace
        config = dataprovider.configuration.filtered_choice(areas=1)
        logical.configlet.configure(config)
        
        # Always logout before logging in
        logical.application.logout()
        user1 = dataprovider.user.take("Sample Manager")
        logical.application.login(user1)
                
        dataprovider.navigation.current = self.context

        logical.folder.delete_using_url(self.groupspace)
        logical.folder.add(self.groupspace)
        logical.content.edit(self.groupspace)
        logical.content.save(self.groupspace)
        
        dataprovider.groupspaceuser.populate()
        
        logical.navigation.down(self.area)
         
    def goto(self):
        dataprovider.navigation.current = self.context
        logical.navigation.down(self.groupspace)

for area in dataprovider.areas.all(): 
    areagroupspace = dataprovider.areagroupspaces.take('GroupSpace with Areas')
    dataprovider.context.provide(id="Area " + area['title'] + " of " + areagroupspace['title'],
                                 setup=SetUpAreaGroupSpaceArea(areagroupspace, area),) 
