# -*- coding: utf-8 -*-
## GrufSpaces
## Group environments for elaborating and exchanging information
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
$Id: $
"""
__author__  = ''
__docformat__ = 'restructuredtext'

# Python imports
from AccessControl import ClassSecurityInfo

# Email support
import email
from email import Encoders
from email.Message import Message
from email.Header import Header
from email.MIMEMultipart import MIMEMultipart
from email.MIMEText import MIMEText
try:
    set
except NameError:
    from sets import Set as set

# CMF imports
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName

# Archetypes imports
try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

# Products imports
from Products.GrufSpaces.config import PROJECTNAME

# GrufSpaces imports
from Products.GrufSpaces.types.schemata import GroupSpaceSchema
from Products.GrufSpaces.permissions import GroupSpace_assignRolePermission
from Products.GrufSpaces.permissions import GroupSpace_sendMailPermission


class GroupSpace(OrderedBaseFolder):
    """GroupSpace container"""
    
    meta_type      = 'ATGroupSpace'
    portal_type    = 'GroupSpace'
    archetype_name = 'GroupSpace'
    immediate_view = 'GroupSpace_listing'
    default_view   = 'GroupSpace_listing'
    global_allow = True
    schema =  GroupSpaceSchema
    content_icon = 'GroupSpace.gif'
    security = ClassSecurityInfo()

    actions = (
        {
        'id'            : 'view',
        'name'          : 'View',
        'action'        : 'string:${object_url}/GroupSpace_listing',
        'permissions'   : (CMFCorePermissions.View, ),
        'category'      : 'object',
        'visible'       : 1,
        },
        {
        'id'            : 'roles',
        'name'          : 'Roles',
        'action'        : 'string:${folder_url}/GroupSpace_rolesForm',
        'permissions'   : (GroupSpace_assignRolePermission,),
        'category'      : 'object',
        'visible'       : 1,
        },
        {
        'id'            : 'email',
        'name'          : 'Email',
        'action'        : 'string:${folder_url}/GroupSpace_emailForm',
        'permissions'   : (GroupSpace_sendMailPermission,),
        'category'      : 'object',
        'visible'       : 1,
        },
        {
        'id'            : 'local_roles',
        'name'          : 'Sharing',
        'action'        : 'string:${folder_url}/folder_localrole_form',
        'permissions'   : (GroupSpace_assignRolePermission,),
        'category'      : 'object',
        'visible'       : 1,
        },
        )    

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        OrderedBaseFolder.manage_afterAdd(self, item, container)
        # Hook for adding your own services in Python Scripts
        self.add_groupspace_services()

    # Security comments:
    #   GroupMembers adding / deletion is protected by the ManageProperties permission.
    #   Members listing is allowed upon AccessContentsInformation permission
    # Sendmail method is taken from mxmWorkgroups

    def _site_encoding(self):
        "Returns the site encoding"
        portal_properties = getToolByName(self, 'portal_properties')    
        site_props = portal_properties.site_properties
        return site_props.default_charset or 'utf-8'

    def _mailhost(self):
        # tries to fetch a mailhost/smtp server url from several locations
        # try site setting
        if hasattr(self, 'MailHost'):
            return self.MailHost

    security.declareProtected(GroupSpace_sendMailPermission, 'sendMail')
    def GroupSpace_mailFormAction(self, to=None, cc=None, bcc=None, inReplyTo=None, 
                                 subject=None, body='', attachments=None, mfrom=None,
                                 REQUEST=None):
        """
        Sends a message. Many of the input parameters are not currently used, 
        but can be used for skinning the functionlity.
        """
        site_encoding = self._site_encoding()
        ##################
        # Create the message
        msg = Message()
        msg.set_payload(body, site_encoding)
        #####################################
        # if attachment, convert to multipart
        # file fields are posted even if empty, so we need to remove those :-s
        if attachments is None:
            attachments = []
        attachments = [a for a in attachments if a] 
        if attachments:
            mimeMsg = MIMEMultipart()
            mimeMsg.attach(msg)
            for attachment in attachments:
                # Add the attachment
                tmp = email.message_from_string(str(attachment.headers))
                filename = tmp.get_param('filename', 'Attachment', 'Content-Disposition')
                # clean up IE paths
                filename = filename[max(filename.rfind('/'),
                                        filename.rfind('\\'),
                                        filename.rfind(':')
                                       )+1:]
                contentType = tmp['Content-Type']
                attach_part = Message()
                attach_part.add_header('Content-Type', contentType, name=filename)
                attach_part.add_header('Content-Disposition', 'attachment', filename=filename)                
                attach_part.set_payload(attachment.read())
                Encoders.encode_base64(attach_part)
                mimeMsg.attach(attach_part)
            msg = mimeMsg
        ########################
        # set headers on message
        ####
        if to is None:
            to = []
        if mfrom:
            msg['From'] = mfrom
            msg['Reply-To'] = mfrom
        to = ','.join(to)
        if to:
            msg['To'] = to
        ####
        if cc is None:
            cc = []
        cc = ','.join(cc)
        if cc:
            msg['Cc'] = cc
        ####
        if bcc is None:
            bcc = []
        bcc = ','.join(bcc)
        if bcc:
            msg['Bcc'] = bcc
        ####
        msg['Date'] = self.ZopeTime().rfc822() # needed by some servers
        if inReplyTo:
            msg['In-Reply-To'] = inReplyTo
        msg['Subject'] = Header(subject, site_encoding)
        ##################
        # Send the message
        SMTPserver = self._mailhost()
        success = 0
        
        
        cleaner = lambda adresses: [adress.strip() for adress in adresses.split(',') if adress.strip()]
        all_receivers = cleaner(to) + cleaner(cc) + cleaner(bcc)
        all_receivers = list(set(all_receivers))
        if all_receivers: # only send if any recipients
            self._mailhost().send(str(msg), mto=all_receivers, mfrom=mfrom)
            success = 1
   
        return success
        
    security.declareProtected(GroupSpace_sendMailPermission, "sendMail")
    def sendMail(self, mail_text):
        """
        Send mail from groupspace
        """
        
        host = self.MailHost
        sent=1
        try:
            host.send(mail_text)
        except:
            sent=0
        return sent

    security.declareProtected(GroupSpace_sendMailPermission, "getMail")
    def getMail(self, member_id):
        # Special utility method needed because of permission restrictions.
        # GroupAdmins don't have the 'Manage Users' permission, so calling
        # getMemberById on portal_membership doesn't work in a Python Script.
        mtool = getToolByName(self, 'portal_membership')
        member = mtool.getMemberById(member_id)
        try:
            email = member.getProperty('email')
        except:
            email=None
        return email

    security.declareProtected(CMFCorePermissions.AccessContentsInformation, "listGroupSpaceMemberIds")
    def listGroupSpaceMemberIds(self):
        """
        listGroupSpaceMemberIds(self,) => Return a list of member ids for this groupspace.
        This will return both users and groups.
        """
        ids = self.users_with_local_role("GroupMember")
        ids.extend(self.users_with_local_role("GroupAdmin"))
        ids.extend(self.users_with_local_role("GroupLeader"))
        return ids

    security.declareProtected(CMFCorePermissions.AccessContentsInformation, "listGroupSpaceVisitorIds")
    def listGroupSpaceVisitorIds(self):
        """
        listGroupSpaceVisitorIds(self,) => Return a list of visitor ids for this groupspace.
        This will return both users and groups.
        """
        return self.users_with_local_role("GroupVisitor")

    security.declareProtected(CMFCorePermissions.AccessContentsInformation, "getMainOwner")
    def getMainOwner(self,):
        """
        getMainOwner(self,) => Return the main owner of the group space.
        It's likely that this will be the person in charge of maintaining it ! :-)
        """
        return self.listGroupOwners()[0]


    security.declareProtected(CMFCorePermissions.AccessContentsInformation, "listGroupOwners")
    def listGroupOwners(self,):
        """
        listGroupOwners(self,) => Return a list of all persons having 'Owner' role in this group space.
        """
        ret = self.users_with_local_role("Owner")

        # XXX TODO:
        # XXX HAVE TO EXTEND THIS LIST TO ALL PEOPLE HAVING THE 'OWNER' ROLE,
        # XXX EVEN IF IT'S NOT A LOCAL ROLE !
        return ret

    #                                           #
    #           Various utilities               #
    #                                           #

    security.declarePublic("show_id")
    def show_id(self,):
        """
        Determine whether to show an id in an edit form

        show_id.py used to be a Python Script, but was removed from Plone 2.5,
        so we have to add the method here for compatability
        """
        if self.plone_utils.isIDAutoGenerated(self.REQUEST.get('id', None) or self.getId()):
            return not (self.portal_factory.isTemporary(self) or self.CreationDate() == self.ModificationDate())
        return 1

    security.declarePublic("canChange")
    def canChange(self,):
        """
        canChange(self,) => return true if the current user can change things in here
        """
        mtool = getToolByName(self, 'portal_membership')
        return mtool.checkPermission(CMFCorePermissions.ManageProperties, self)

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, "setInitialGroup")
    def setInitialGroup(self, group):
        # The group that we create the workspace for is automatically
        # becoming GroupMember
        self.manage_setLocalRoles(group.getId(), ['GroupMember'])

    #                                                                   #
    #           CollabSpace Interface                                   #
    #                                                                   #
    # Implementation of the CollabSpace Interface as defined in         #
    # PloneGroupSpace by Helmut Merz:                                   #
    #                                                                   #
    # http://cvs.sourceforge.net/viewcvs.py/collective/PloneGroupSpace/ # 
    #                                                                   #

    def getValidRoles(self):
        """ May be used to show a list of roles to select from in the user interface"""
        return ('GroupVisitor', 'GroupMember', 'GroupAdmin', 'GroupLeader')

    def getCollabRoles(self):
        return ('GroupVisitor', 'GroupMember', 'GroupAdmin', 'GroupLeader')

    def getCollabUsers(self, role=None):
        # Return all users with a local role
        if role is None:
            # Get Collab Users for all roles
            checkRoles = self.getCollabRoles()
        else:
            # Get Collab Users only for the given role
            checkRoles = [role]
        result = []
        lr = self.get_local_roles()
        for (user, roles) in lr:
            for r in roles:
                if r in checkRoles:
                    result.append(user)
                    break # One role is enough
        return tuple(result)

    def getCollabUserObjects(self, role=None):
        """ Returns real user objects - Member or Group - not just the ID.
        """
        userids = self.getCollabUsers(role)
        pg = getToolByName(self, 'portal_groups')
        pm = getToolByName(self, 'portal_membership')
        result = []
        for userid in userids:
            obj = pg.getGroupById(userid)
            if obj:
                user_type = 'group'
            else:
                obj = pm.getMemberById(userid)
                user_type = 'user'
            if obj:
                result.append( (user_type, obj) )
        return result

    security.declareProtected(GroupSpace_assignRolePermission, 'assignCollabRoleToUser')
    def assignCollabRoleToUser(self, role, userid):
        if role not in self.getCollabRoles():
            raise ValueError, 'Role %s is not registered as a Collab Role here.' % role
        pm = getToolByName(self, 'portal_membership')
        user = pm.getMemberById(userid)
        if not user:
            raise ValueError, 'Member %s does not exist.' % userid
        self.manage_addLocalRoles(user.getId(), [role])
        self.reindexObjectSecurity()
        self.reindexObject(idxs=['listGroupSpaceMemberIds'])
        self.reindexObject(idxs=['listGroupSpaceVisitorIds'])
        self._afterAssignToCollabRole(role, user.getId(), user_type="user")

    security.declareProtected(GroupSpace_assignRolePermission, 'assignCollabRoleToGroup')
    def assignCollabRoleToGroup(self, role, groupid):
        if role not in self.getCollabRoles():
            raise ValueError, 'Role %s is not registered as a Collab Role here.' % role
        pg = getToolByName(self, 'portal_groups')
        group = pg.getGroupById(groupid)
        if not group:
            raise ValueError, 'Group %s does not exist.' % userid
        self.manage_addLocalRoles(group.getId(), [role])
        self.reindexObjectSecurity()
        self.reindexObject(idxs=['listGroupSpaceMemberIds'])
        self.reindexObject(idxs=['listGroupSpaceVisitorIds'])
        self._afterAssignToCollabRole(role, group.getId(), user_type="group")

    def getCollabUserDictionary(self):
        """ A convenience function to get the full information on assigned
            users with one method call. Returns a dictionary with roles as keys and
            a tuple of assigned users as the values.
        """
        result = {}
        for role in self.getCollabRoles():
            users = self.getCollabUsers(role)
            if users:
                result[role] = users
        return result

    security.declareProtected(GroupSpace_assignRolePermission, 'removeCollabRoleFromUser')
    def removeCollabRoleFromUser(self, role, userid):
        pm = getToolByName(self, 'portal_membership')
        member = pm.getMemberById(userid)
        if not member:
            raise ValueError, 'Member %s does not exist.' % userid
        lr = []
        try:
            lr = list(self.get_local_roles_for_userid(member.getId()))
        except:
            pass
        if role in lr:
            lr.remove(role)
        if lr:
            self.manage_setLocalRoles(member.getId(), lr)
        else:
            try:
                self.manage_delLocalRoles([member.getId()])
            except:
                pass
        self.reindexObjectSecurity()
        self.reindexObject(idxs=['listGroupSpaceMemberIds'])
        self.reindexObject(idxs=['listGroupSpaceVisitorIds'])
        self._afterRemoveFromCollabRole(role, member.getId(), user_type="group")

    security.declareProtected(GroupSpace_assignRolePermission, 'removeCollabRoleFromGroup')
    def removeCollabRoleFromGroup(self, role, groupid):
        pg = getToolByName(self, 'portal_groups')
        group = pg.getGroupById(groupid)
        if not group:
            raise ValueError, 'Group %s does not exist.' % groupid
        lr = []
        try:
            lr = list(self.get_local_roles_for_userid(group.getId()))
        except:
            pass
        if role in lr:
            lr.remove(role)
        if lr:
            self.manage_setLocalRoles(group.getId(), lr)
        else:
            try:
                self.manage_delLocalRoles([group.getId()])
            except:
                pass
        self.reindexObjectSecurity()
        self.reindexObject(idxs=['listGroupSpaceMemberIds'])
        self.reindexObject(idxs=['listGroupSpaceVisitorIds'])
        self._afterRemoveFromCollabRole(role, group.getId(), user_type="group")

    def _afterAssignToCollabRole(self, role, user, user_type):
        """ This method may be overridden in subclasses of GroupSpace,
            e.g. to register the assignment with the Member or Group object assigned.
        """
        pass

    def _afterRemoveFromCollabRole(self, role, user, user_type):
        """ This method may be overridden in subclasses of GroupSpace,
            e.g. to remove the assignment from the Member or Group object assigned.
        """
        pass

    security.declareProtected(CMFCorePermissions.View, 'getCurrentGroupspace')
    def getCurrentGroupspace(self):
        """getCurrentGroupspace(self, ) => return itself
        """
        return self.restrictedTraverse('')

    #                                                                   #
    #           Extensions to the CollabSpace Interface                 #
    #                                                                   #

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, "getMainCollabUserRole")
    def getMainCollabUserRole(self, user):
        # Return the main role of a user to be shown in the interface.
        roles= user.getRolesInContext(self)
        role = None
        if 'GroupLeader' in roles:
            role = 'GroupLeader'
        elif 'GroupAdmin'in roles:
            role = 'GroupAdmin'
        elif 'GroupMember' in roles:
            role = 'GroupMember'
        elif 'GroupVisitor' in roles:
            role = 'GroupVisitor'
        return role

        
registerType(GroupSpace, PROJECTNAME)
